from orchestrator.forms import FormPage
from orchestrator.forms.validators import Label
from orchestrator.targets import Target
from orchestrator.types import FormGenerator, SubscriptionLifecycle, UUIDstr
from orchestrator.workflow import StepList, done, init, workflow
from orchestrator.workflows.steps import resync, set_status, store_process_subscription, unsync
from orchestrator.workflows.utils import wrap_modify_initial_input_form

from gso.products.product_types.site import Site


def initial_input_form_generator(subscription_id: UUIDstr) -> FormGenerator:
    Site.from_subscription(subscription_id)

    class TerminateForm(FormPage):
        termination_label: Label = "Are you sure you want to delete this site?"  # type: ignore[assignment]

    user_input = yield TerminateForm
    return user_input.dict()


@workflow(
    "Terminate Site",
    initial_input_form=wrap_modify_initial_input_form(initial_input_form_generator),
    target=Target.TERMINATE,
)
def terminate_site() -> StepList:
    return (
        init
        >> store_process_subscription(Target.TERMINATE)
        >> unsync
        >> set_status(SubscriptionLifecycle.TERMINATED)
        >> resync
        >> done
    )
